#include "T2.hh"
#include <Ice/Ice.h>
#include "interrupt.hh"
#include "srv.h"
#include <sstream>
#include <unistd.h>

//----------------------------------------------------------------------
T2::T2(Interrupt * i,
       const std::string & hs_addr)
  : i_(i), hs_addr_(hs_addr)
{
}

//----------------------------------------------------------------------
void T2::run()
{

  // Create new communicator and get properties.

  int argc = 1;
  char * argv[] = { "T2", 0 };
  Ice::CommunicatorPtr ic = Ice::initialize(argc, argv);

  //Ice::Trace log_trc(ic->getLogger(), "T2");

  //log_trc << "T2 thread started.";
  //log_trc.flush();

  MG::HSPrx hs;
  try {
    Ice::ObjectPrx obj = ic->stringToProxy(hs_addr_);
    hs = MG::HSPrx::checkedCast(obj);

    // BEGIN

    hs->begin();
  }

  catch ( const Ice::Exception & e ) {
    std::ostringstream smsg;
    smsg << "T2: Begin error:\n" << e;
    ic->getLogger()->error(smsg.str());
    assert(false);
    throw smsg.str();
  }

  //log_trc << "BEGIN";
  //log_trc.flush();

  // Infinite loop

  while ( ! i_->interrupted() ) {
    sleep(1);
  }

  // END

  try {
    sleep(2); // to sync with T1
    hs->end();
    //log_trc << "END";
    //log_trc.flush();
  }
  catch ( const Ice::Exception & e ) {
    std::ostringstream smsg;
    smsg << "T2: END error:\n" << e;
    ic->getLogger()->error(smsg.str());
  }


  // Destroy communicator.

  if ( ic ) ic->destroy();
}
