#include <Ice/Ice.h>
#include "interrupt.hh"
#include "T1.hh"
#include "T2.hh"
#include <iostream>
using std::cout;
using std::endl;
using std::string;

namespace {
  Interrupt * i_;
  IceUtil::ThreadPtr t1_;
  IceUtil::ThreadPtr t2_;
}


//--------------------------------------------------------------------
bool stop(const Ice::CommunicatorPtr & ic)
{
  Ice::Trace out(ic->getLogger(), "CE");
  i_->setInterrupted();

  t1_->getThreadControl().join();
  out << "T1 finished.";
  out.flush();

  t2_->getThreadControl().join();
  out << "T2 finished.";
  out.flush();

  out << "client stopped.";
  out.flush();
  return true;
}

//--------------------------------------------------------------------
int
run(int argc, char* argv[], const Ice::CommunicatorPtr & ic)
{
  try {
    i_ = new Interrupt;

    // Activate Thread 1

    t1_ = new T1(i_);
    t1_->start();

    // Activate Thread 2

    t2_ = new T2(i_, "HS:default -p 13004");
    t2_->start();

    // Activate the Servant.

    char c;
    for(;;) {
      std::cout << "> ";
      std::cin >> c;
      if ( c == 's' ) {
	stop(ic);
	break;
      }
    }
  }

  catch ( const std::string & msg ) {
    ic->getLogger()->error(msg);
    return false;
  }

  return true;
}

//--------------------------------------------------------------------
int main(int argc, char * argv[])
{
  int status;
  Ice::CommunicatorPtr communicator;

  try {
    communicator = Ice::initialize(argc, argv);
    status = run(argc, argv, communicator);
  }
  catch(const Ice::Exception& ex) {
    std::cerr << ex << std::endl;
    status = EXIT_FAILURE;
  }

  if(communicator) {
    try {
      communicator->destroy();
    }
    catch(const Ice::Exception& ex) {
      std::cerr << ex << std::endl;
      status = EXIT_FAILURE;
    }
  }

  return status;
}
